@@ -1,7 +1,9 @@
 # coding=utf-8
 import configparser
+import json
 import os
 import shutil
+from typing import Optional
 
 import click
 import git
@@ -11,6 +13,7 @@ import csv
 from glob import glob
 
 from result import Ok, Err, Result
+from semver import VersionInfo
 
 KEY_STUDENT_ID = 'id'
 KEY_STUDENT_NAME_FIRST = 'first_name'
@@ -32,6 +35,7 @@ EVAL_COMMAND_NAME = 'ep2_eval'
 KEY_UE_NO = 'ue'
 KEY_UE_TAG = 'tag'
 
+__version__ = '0.4.9'
 
 def relative_path(basedir, filename):
     """Creates a path relative to base"""
@@ -206,6 +210,8 @@ class Ep2Group:
 class Ep2:
 
     def __init__(self, verbose: bool = False, config: configparser.ConfigParser = None, sudo: str = None):
+        from getversion import get_module_version
+        import ep2_core
         if config is None:
             self.config = create_config()
         else:
@@ -218,6 +224,45 @@ class Ep2:
         if self.sudo is not None:
             self.print_sudo_header()
 
+        version, details = get_module_version(ep2_core.common)
+
+        if verbose:
+            click.secho("Starting tool version {}".format(version), fg='bright_black')
+            click.secho("Fetching latest release version from pypi", fg='bright_black')
+
+        online_version = self.fetch_pypi_version()
+        local_version = VersionInfo.parse(version)
+
+        if online_version.is_ok():  # value is version
+            if online_version.value > local_version:
+                click.secho('You are using an outdated version ({}) of ep2-tutor-tools'.format(local_version), fg='yellow')
+                click.secho('Some commands might not work as expected!'.format(local_version), fg='yellow')
+                click.secho('Please update the tools by running:', fg='yellow')
+                click.echo()
+                click.secho('    pip install -U --user ep2-tutor-scripts=={}'.format(online_version.value), fg='yellow', bold=True)
+                click.echo()
+            elif verbose:
+                click.secho('Tools are up to date', fg='bright_black')
+        else:   # value is str
+            click.secho("Could not fetch latest version from pypi: {}".format(online_version.value), fg='red', bold=True)
+
+    def fetch_pypi_version(self) -> Result[VersionInfo, str]:
+        import requests
+
+        response = requests.get('https://pypi.org/pypi/ep2-tutor-scripts/json')
+        if response.status_code != 200:
+            return Err('HTTP error: {}'.format(response.status_code))
+
+        try:
+            data = response.json()
+            version = data['info']['version']
+            version = VersionInfo.parse(version)
+            return Ok(version)
+        except json.JSONDecodeError as e:
+            return Err('JSON Decode Error: {}'.format(e))
+        except Exception as e:
+            return Err('Other error: {}'.format(e))
+
     def tag_project(self, project, tag):
         """Tags the last commit of a given project with 'tag'."""
         try:  # search for project
@@ -247,6 +292,11 @@ class Ep2:
 
         return True
 
+    def git_host(self) -> str:
+        from urllib.parse import urlparse
+        url = urlparse(self.config['Gitlab']['url'])
+        return url.hostname
+
     def clone_or_update(self, repo_dir, project, tag) -> Result:
         """Clones or updates a repository into repo_dir"""
 
@@ -304,6 +354,13 @@ class Ep2:
         """Returns the local path of students uebungs repo"""
         return os.path.join(self.config.get('Local', 'GitHome'), 'uebung', mat_no)
 
+    def ex_test_repo_name(self, mat_no: str, ue: int):
+        return '{}u{}'.format(mat_no, ue)
+
+    def local_ex_test_repo(self, mat_no: str, ue: int):
+        """Returns the local path of students uebungs repo"""
+        return os.path.join(self.config.get('Local', 'GitHome'), 'uebung', self.ex_test_repo_name(mat_no, ue))
+
     def tutor_repo(self):
         """Returns the path to the local tutor directory"""
         return self.config.get('Local', 'TutorRepo',
@@ -524,6 +581,46 @@ class Ep2:
 
         return True, grading
 
+    def exercise_test_repo(self, ex: int, student_id: str) -> Optional[str]:
+        ex_type = ex_test_type(ex)
+
+        if ex_type == ExTestType.Single:
+            return os.path.join(self.config.get("Gitlab", "RepoPrefix"), 'uebung', self.ex_test_repo_name(student_id, ex)).replace("\\", "/")
+        else:
+            return None
+
+    def grant_access(self, project_path: str, student_id: str) -> Result[None, str]:
+        """ Grants maintainer access for `student_id` to the project at `project_path`.
+
+        Maintainer access level is necessary to be able to push to `master`.
+
+        :param project_path: the path to the project (e.g. `<course>/<term>/uebung/<student>
+        :param student_id: the student id of the student
+        :return: `Ok(None)` if the operation succeeds, `Err(error_msg)` if an error occurs
+        """
+
+        try:
+            project = self.gitlab.projects.get(project_path.replace("/", "%2F"))
+            user = self.gitlab.users.list(username=student_id)[0]
+
+            members = project.members.list()    # lists the directly assigned members (should not need another request)
+            if any([member.id == user.id for member in members]):   # member already exists
+                member = project.members.get(user.id)
+                if member.access_level < gitlab.MAINTAINER_ACCESS:
+                    click.secho('setting access level of {} to maintainer for project {}'
+                                .format(student_id, project_path), fg='bright_black')
+                    member.access_level = gitlab.MAINTAINER_ACCESS
+                    member.save()
+            else:   # create member
+                project.members.create({
+                    'user_id': user.id,
+                    'access_level': gitlab.MAINTAINER_ACCESS,
+                })
+
+            return Ok(None)
+        except Exception as e:
+            return Err('granting {} maintainer access to {} failed: {}'.format(student_id, project_path, e))
+
 
 def verify_and_normalize_student_id(student_id) -> Result:
     import re
@@ -580,7 +677,7 @@ def create_config():
 
 
 def ue_type(ue: int):
-    if int(ue) == 7:
+    if int(ue) == 6:
         return ExerciseType.Team
     return ExerciseType.Normal
 
