@@ -5,6 +5,8 @@ from __future__ import print_function
 import random
 import string
 
+import click
+
 from ep2_core.common import *
 
 from Cheetah.Template import Template
@@ -172,13 +174,50 @@ def team_name(members):
 @click.option("--group", required=True, prompt=True, help='name of the group')
 @click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', type=click.INT)
 @click.option("--late/--on-time", default=False, help='if submission was late')
-@click.confirmation_option(prompt='This will tag all repositories with the adhoc tags. Continue?')
+@click.confirmation_option(prompt='This will tag all repositories of the group with the exercise test tags. Continue?')
 @click.pass_context
 def tag(ctx, group, ue: int, late):
     ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
 
-    group.tag_group((TAG_NAME_EX_TEST % ue) if not late else (TAG_NAME_EX_TEST_LATE % ue))
+    ex_type = ex_test_type(ue)
+
+    if ex_type == ExTestType.Team:
+        group.tag_group((TAG_NAME_EX_TEST % ue) if not late else (TAG_NAME_EX_TEST_LATE % ue))
+    else:
+        students = group.student_list()
+        errors = []
+
+        with click.progressbar(students, label='Tagging projects', show_eta=True) as bar:
+            for student in bar:
+                ep2.tag_project(ep2.exercise_test_repo(ue, student),
+                                (TAG_NAME_EX_TEST % ue) if not late else (TAG_NAME_EX_TEST_LATE % ue))
+
+
+@cli.command()
+@click.option("--group", required=True, prompt=True, help='name of the group')
+@click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', type=click.INT)
+@click.confirmation_option(prompt='This will give students access to the repository for the exercise test. Continue?')
+@click.pass_context
+def grant_access(ctx, group, ue: int):
+    ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
+    group = ep2.group(group)
+
+    students = group.student_list()
+    errors = []
+
+    with click.progressbar(students, label='Adding members', show_eta=True) as bar:
+        for student in bar:
+            result = ep2.grant_access(ep2.exercise_test_repo(ue, student), student)
+            if result.is_err():
+                errors += [result.value]
+
+    if len(errors) > 0:
+        click.secho('Granting access completed with errors:', fg='red', bold=True)
+        for error in errors:
+            click.secho('    ' + error, fg='red', bold=True)
+    else:
+        click.secho('Granting access completed successfully', fg='green', bold=True)
 
 
 @cli.command()
@@ -188,18 +227,14 @@ def tag(ctx, group, ue: int, late):
 @click.option("--idea/--no-idea", default=False,
               help="load project into the folder, that is monitored by IntelliJ Idea")
 @click.option("--grade/--no-grade", default=False, help="also add grading with submission")
-@click.option("--single/--team", default=False, help="type of the exercise, default is team")
 @click.pass_context
-def submission(ctx, group, ue, late, idea, grade, single):
+def submission(ctx, group, ue, late, idea, grade):
     """Adds submissions of teams for a specific exercise to the corresponding CSV files."""
     ep2 = Ep2(verbose=ctx.obj["VERBOSE"], sudo=ctx.obj["SUDO"])
     group = ep2.group(group)
     f_info = FileInformation('Submission')
 
-    curr_adhoc_type = ex_test_type(ue)
-    if curr_adhoc_type != (ExTestType.Single if single else ExTestType.Team):
-        click.confirm('You have chosen exercise type %s even tough ad hoc exercise %s is marked as %s. Continue?' %
-                      ('Single' if single else 'Team', ue, curr_adhoc_type.name), abort=True)
+    ex_type = ex_test_type(ue)
 
     index = TeamIndex(group)
 
@@ -215,7 +250,7 @@ def submission(ctx, group, ue, late, idea, grade, single):
 
     f_info.open_write(csv_file, True)
 
-    if not single:
+    if ex_type == ExTestType.Team:
         if file_exists:
             with open(csv_file, 'r') as csvfile:
                 reader = csv.DictReader(csvfile, ex_test_csv_fieldnames(), KEY_INVALID, strict=True)
@@ -302,10 +337,10 @@ def submission(ctx, group, ue, late, idea, grade, single):
                     members = [(repo_owner, 'o'), (editor, 'e')]
 
                 if grade:
-                    current_uebung = ep2.local_repo(repo_owner)
+                    local_repo = ep2.local_repo(repo_owner)
 
-                    if ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST_LATE % ue).is_err():
-                        clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST % ue)
+                    if ep2.clone_or_update(local_repo, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST_LATE % ue).is_err():
+                        clone_result = ep2.clone_or_update(local_repo, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST % ue)
                         if clone_result.is_err():
                             click.secho("Could not checkout submission from {} for team {}, {}"
                                         .format(repo_owner, team.name, clone_result.value), fg='red')
@@ -353,23 +388,24 @@ def submission(ctx, group, ue, late, idea, grade, single):
                 if action == 'a':
                     for student in sorted(index.students):
                         if grade:
-                            current_uebung = ep2.local_repo(student)
+                            local_repo = ep2.local_ex_test_repo(student, ue)
+                            repo_name = ep2.ex_test_repo_name(student, ue)
 
-                            if ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                            if ep2.clone_or_update(local_repo, ep2.exercise_test_repo(ue, student),
                                                        TAG_NAME_EX_TEST_LATE % ue).is_err():
-                                clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                clone_result = ep2.clone_or_update(local_repo, ep2.exercise_test_repo(ue, student),
                                                                    TAG_NAME_EX_TEST % ue)
                                 if clone_result.is_err():
                                     click.secho("Could not checkout submission from {}, {}"
-                                                .format(student, clone_result.value), fg='red')
+                                                .format(repo_name, clone_result.value), fg='red')
                                 else:
                                     if idea:
-                                        ep2.idea_checkout(student)
-                                        click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+                                        ep2.idea_checkout(repo_name)
+                                        click.secho('Project %s loaded into IntelliJ Idea' % repo_name, fg='green')
                             else:
                                 if idea:
-                                    ep2.idea_checkout(student)
-                                    click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+                                    ep2.idea_checkout(repo_name)
+                                    click.secho('Project %s loaded into IntelliJ Idea' % repo_name, fg='green')
 
                             points = click.prompt("Points for {}".format(student), type=click.IntRange(0, 4),
                                                   show_choices=True)
@@ -389,25 +425,27 @@ def submission(ctx, group, ue, late, idea, grade, single):
                     for student in sorted(index.students):
                         if click.confirm('Add submission for student {}?'.format(student)):
                             if grade:
-                                current_uebung = ep2.local_repo(student)
+                                local_repo = ep2.local_ex_test_repo(student, ue)
+                                repo_name = ep2.ex_test_repo_name(student, ue)
 
-                                if ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
-                                                           TAG_NAME_EX_TEST_LATE % ue).is_err():
-                                    clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                if ep2.clone_or_update(local_repo, ep2.exercise_test_repo(ue, student),
+                                                       TAG_NAME_EX_TEST_LATE % ue).is_err():
+                                    clone_result = ep2.clone_or_update(local_repo, ep2.exercise_test_repo(ue, student),
                                                                        TAG_NAME_EX_TEST % ue)
                                     if clone_result.is_err():
                                         click.secho("Could not checkout submission from {}, {}"
-                                                    .format(student, clone_result.value), fg='red')
+                                                    .format(repo_name, clone_result.value), fg='red')
                                     else:
                                         if idea:
-                                            ep2.idea_checkout(student)
-                                            click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+                                            ep2.idea_checkout(repo_name)
+                                            click.secho('Project %s loaded into IntelliJ Idea' % repo_name, fg='green')
                                 else:
                                     if idea:
-                                        ep2.idea_checkout(student)
-                                        click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+                                        ep2.idea_checkout(repo_name)
+                                        click.secho('Project %s loaded into IntelliJ Idea' % repo_name, fg='green')
 
-                                points = click.prompt("Points", type=click.IntRange(0, 4), show_choices=True)
+                                points = click.prompt("Points for {}".format(student), type=click.IntRange(0, 4),
+                                                      show_choices=True)
                                 remarks = click.prompt("Remarks", default='', show_default=True)
                                 feedback = click.prompt("Feedback", default='', show_default=True)
                             else:
